/*
 Andrew Orlov, andrew.v.orlov@gmail.com, 26/02/2009
 Important notes:
 we suppose that this code will be used with MySQL DBMS.
*/

-- Server settings
CREATE TABLE ic_systemsettings (
    paramname VARCHAR(255) UNIQUE NOT NULL, -- parameter name (key)
    paramval LONGTEXT NOT NULL, -- parameter value
    PRIMARY KEY (paramname)
) ENGINE=InnoDB;

INSERT INTO ic_systemsettings (paramname, paramval) VALUES ('DATABASE_REVISION', '2');

-- LDAP authentication sources
CREATE TABLE ic_ldap (
    id INTEGER UNIQUE NOT NULL, -- message type ID
    ldapdomain VARCHAR(255) UNIQUE NOT NULL, -- domain name
    ldapurl VARCHAR(255) NOT NULL, -- LDAP url
    ldapprincipalformat VARCHAR(255) NOT NULL, -- LDAP principal format
    ldapcnattribute VARCHAR(255) NOT NULL, -- user attribute storing displayed name (like cn)
    ldaploginattribute VARCHAR(255) NOT NULL, -- user attribute storing login (like sAMAccountName in AD or uid in posixAccount)
    ldapmappingattribute VARCHAR(255) NOT NULL, -- unique user attribute to map IC user with LDAP user (like objectGUID in AD or uidNumber in posixAccount)
    ldapisbinaryattribute BOOLEAN NOT NULL DEFAULT FALSE, -- is mapping attribute binary or not?
    ldapsearchbase VARCHAR(255) NOT NULL, -- base to search in if it differs from ldapdomain
    ldapfilter VARCHAR(255) NOT NULL, -- filter to search users
    PRIMARY KEY (id)
) ENGINE=InnoDB;

-- Templates
CREATE TABLE ic_templates (
    templatename VARCHAR(255) UNIQUE NOT NULL, -- template name
    PRIMARY KEY (templatename)
) ENGINE=InnoDB;

INSERT INTO ic_templates (templatename) VALUES ('IC_FILES');
INSERT INTO ic_templates (templatename) VALUES ('IC_MESSAGES');

-- Modules
CREATE TABLE ic_modules (
    modulename VARCHAR(255) UNIQUE NOT NULL, -- module name
    PRIMARY KEY (modulename)
) ENGINE=InnoDB;

INSERT INTO ic_modules (modulename) VALUES ('IC_JOURNAL');
INSERT INTO ic_modules (modulename) VALUES ('IC_ADMINISTRATION');
INSERT INTO ic_modules (modulename) VALUES ('IC_USERMANAGEMENT');
INSERT INTO ic_modules (modulename) VALUES ('IC_SHAREDLISTS');

-- Permissions
CREATE TABLE ic_permissions (
    permission VARCHAR(255) UNIQUE NOT NULL, -- permission name
    PRIMARY KEY (permission)
) ENGINE=InnoDB;

INSERT INTO ic_permissions (permission) VALUES ('IC_NONE');
INSERT INTO ic_permissions (permission) VALUES ('IC_READ');
INSERT INTO ic_permissions (permission) VALUES ('IC_WRITE');

-- Groups
CREATE TABLE ic_groups (
    id INTEGER UNIQUE NOT NULL, -- group ID
    groupname VARCHAR(255) NOT NULL, -- group name
    pid INTEGER NOT NULL DEFAULT '-1', -- parent group ID
    PRIMARY KEY (id)
) ENGINE=InnoDB;

INSERT INTO ic_groups (id, groupname, pid) VALUES ('-1', '_TOP_LEVEL_GROUP_', '-1');
INSERT INTO ic_groups (id, groupname, pid) VALUES ('0', 'IT Department', '-1');

ALTER TABLE ic_groups ADD CONSTRAINT ic_groups_pid_fkey FOREIGN KEY (pid) REFERENCES ic_groups (id);

-- Message types
CREATE TABLE ic_messagetypes (
    id INTEGER UNIQUE NOT NULL, -- message type ID
    typename VARCHAR(255) UNIQUE NOT NULL, -- message type name
    templatename VARCHAR(255) NOT NULL REFERENCES ic_templates (templatename), -- template to use
    defaultpermission VARCHAR(255) NOT NULL REFERENCES ic_permissions (permission), -- default permission for this type
    PRIMARY KEY (id)
) ENGINE=InnoDB;

INSERT INTO ic_messagetypes (id, typename, templatename, defaultpermission) VALUES ('0', 'Files', 'IC_FILES', 'IC_WRITE');
INSERT INTO ic_messagetypes (id, typename, templatename, defaultpermission) VALUES ('1', 'Private', 'IC_MESSAGES', 'IC_WRITE');

-- User Data
CREATE TABLE ic_users (
    id INTEGER UNIQUE NOT NULL, -- user UID
    isvalid BOOLEAN DEFAULT TRUE NOT NULL, -- is the user account active or disabled
    username VARCHAR(255) NOT NULL, -- username to show
    timezone VARCHAR(255) NOT NULL, -- user timezone
    locale VARCHAR(255) NOT NULL, -- user locale
    gid INTEGER NOT NULL REFERENCES ic_groups (id), -- user's group ID
-- user properties:
    email VARCHAR(255), -- user's EMail
    phonelocal VARCHAR(255), -- user's internal (local) phone number
    phoneext VARCHAR(255), -- user's external (town) phone number
    phonemob VARCHAR(255), -- user's mobile phone number
    fax VARCHAR(255), -- user's fax number
    birthday DATE, -- user's birthday
    userposition VARCHAR(255), -- user's position
    picture TEXT, -- user's picture/photo, BASE64-encoded 
-- client properties:
    ipint VARCHAR(255), -- User's internal IP during last connection
    ipext VARCHAR(255), -- User's external IP during last connection
    hostname VARCHAR(255), -- User's system name (computer name)
    dnsname VARCHAR(255), -- User's computer DNS name
    clientversion VARCHAR(255), -- version of client used by user
    PRIMARY KEY (id)
) ENGINE=InnoDB;

-- Default password for login=admin is 'admin' (without quotes)
INSERT INTO ic_users (id, username, timezone, locale, gid) VALUES ('0', 'System Administrator', 'UTC', 'en_US', '0');

-- User Auth Data
CREATE TABLE ic_auth_internal (
    uid INTEGER UNIQUE NOT NULL REFERENCES ic_users (id) ON DELETE CASCADE ON UPDATE CASCADE, -- user UID
    userlogin VARCHAR(255) UNIQUE NOT NULL, -- user login
    userpassword VARCHAR(255) NOT NULL, -- user password (md5 hash)
    PRIMARY KEY (userlogin)
) ENGINE=InnoDB;

-- Default password for login=admin is 'admin' (without quotes)
INSERT INTO ic_auth_internal (uid, userlogin, userpassword) VALUES ('0', 'admin', md5('admin'));

-- User Auth Data
CREATE TABLE ic_auth_external (
    uid INTEGER UNIQUE NOT NULL REFERENCES ic_users (id) ON DELETE CASCADE ON UPDATE CASCADE, -- user UID
    ldapmapping VARCHAR(255) NOT NULL, -- value of unique LDAP user attribute
    lid INTEGER NOT NULL REFERENCES ic_ldap (id),
    PRIMARY KEY (ldapmapping, lid)
) ENGINE=InnoDB;

-- Module permissions
CREATE TABLE ic_moduleaccess (
    uid INTEGER NOT NULL REFERENCES ic_users (id), -- user UID
    modulename VARCHAR(255) NOT NULL REFERENCES ic_modules (modulename), -- module name
    moduleaccess BOOLEAN DEFAULT FALSE NOT NULL -- can user access this module or not?
) ENGINE=InnoDB;

INSERT INTO ic_moduleaccess (uid, modulename, moduleaccess) VALUES ('0', 'IC_JOURNAL', TRUE);
INSERT INTO ic_moduleaccess (uid, modulename, moduleaccess) VALUES ('0', 'IC_ADMINISTRATION', TRUE);
INSERT INTO ic_moduleaccess (uid, modulename, moduleaccess) VALUES ('0', 'IC_USERMANAGEMENT', TRUE);
INSERT INTO ic_moduleaccess (uid, modulename, moduleaccess) VALUES ('0', 'IC_SHAREDLISTS', TRUE);

-- Message type permissions
CREATE TABLE ic_messagetypepermissions (
    uid INTEGER NOT NULL REFERENCES ic_users (id), -- user UID
    tid INTEGER NOT NULL REFERENCES ic_messagetypes (id), -- type of the message
    permission VARCHAR(255) NOT NULL REFERENCES ic_permissions (permission), -- user permission to work with this module
    UNIQUE (uid, tid)
) ENGINE=InnoDB;

INSERT INTO ic_messagetypepermissions (uid, tid, permission) VALUES ('0', '0', 'IC_WRITE');
INSERT INTO ic_messagetypepermissions (uid, tid, permission) VALUES ('0', '1', 'IC_WRITE');

-- Messages
CREATE TABLE ic_messages (
    id BIGINT UNIQUE NOT NULL, -- message ID 
    tid INTEGER NOT NULL REFERENCES ic_messagetypes (id), -- type of the message
    mhead LONGTEXT, -- message header
    mbody LONGTEXT, -- message body
    mdate BIGINT NOT NULL, -- UNIX time (UTC) when the message was received by server
    sid INTEGER NOT NULL REFERENCES ic_users (id), -- sender UID
    PRIMARY KEY (id)
) ENGINE=InnoDB;

CREATE INDEX ic_messages_mdate_index ON ic_messages (mdate);
CREATE INDEX ic_messages_sid_index ON ic_messages (sid);

-- Binary data
CREATE TABLE ic_messages_binarydata (
    mid BIGINT NOT NULL REFERENCES ic_messages (id), -- message ID 
    chunk_id BIGINT NOT NULL, -- chunk id
    chunk_data BLOB NOT NULL, -- chunk binary data (BLOB)
    PRIMARY KEY (mid, chunk_id)
) ENGINE=InnoDB;

CREATE INDEX ic_messages_binarydata_mid_index ON ic_messages_binarydata (mid);

-- Message recipients
CREATE TABLE ic_recipients (
    mid BIGINT NOT NULL REFERENCES ic_messages (id), -- message ID
    rid INTEGER NOT NULL REFERENCES ic_users (id), -- recipient UID
    processed BOOLEAN DEFAULT FALSE NOT NULL,  -- is True (1) if message was seen or file was saved, is False (0) if not
    pcomment TEXT, -- processing comment: filename of saved file with full path or some other info (for example, DELETED status etc.)
    pdate BIGINT, -- UNIX time (UTC) when the message was processed (or file was saved)
    PRIMARY KEY (mid, rid)
) ENGINE=InnoDB;

CREATE INDEX ic_recipients_mid_index ON ic_recipients (mid);
CREATE INDEX ic_recipients_rid_index ON ic_recipients (rid);
CREATE INDEX ic_recipients_processed_index ON ic_recipients (processed);

-- User Journal
CREATE TABLE ic_journal (
    uid INTEGER NOT NULL REFERENCES ic_users (id), -- user UID
    _in BIGINT NOT NULL, -- UNIX time (UTC) when user came in session
    _out BIGINT NOT NULL, -- UNIX time (UTC) when user get out session
    PRIMARY KEY (uid, _in)
) ENGINE=InnoDB;

CREATE INDEX ic_journal_uid_index ON ic_journal (uid);
CREATE INDEX ic_journal_in_index ON ic_journal (_in);

-- Recipient Lists
CREATE TABLE ic_recipientlists (
    id INTEGER UNIQUE NOT NULL, -- list ID
    uid INTEGER NOT NULL REFERENCES ic_users (id), -- creator (owner) ID
    listname VARCHAR(255) NOT NULL, -- list name
    isshared BOOLEAN DEFAULT FALSE NOT NULL, --  is True if list is for All, otherwise False
    PRIMARY KEY (id)
) ENGINE=InnoDB;

-- Recipient list members
CREATE TABLE ic_recipientlistmembers (
    lid INTEGER NOT NULL REFERENCES ic_recipientlists (id) ON UPDATE CASCADE ON DELETE CASCADE, -- list ID
    uid INTEGER NOT NULL REFERENCES ic_users (id) ON UPDATE CASCADE ON DELETE CASCADE, -- member ID
    PRIMARY KEY (lid, uid)
) ENGINE=InnoDB;

-- Journal selection
CREATE TABLE ic_journalselection (
    uid INTEGER NOT NULL REFERENCES ic_users (id) ON UPDATE CASCADE ON DELETE CASCADE, -- owner ID
    sid INTEGER NOT NULL REFERENCES ic_users (id) ON UPDATE CASCADE ON DELETE CASCADE, -- selection member ID
    PRIMARY KEY (uid, sid)
) ENGINE=InnoDB;

-- Parameters
CREATE TABLE ic_parameters (
    uid INTEGER NOT NULL REFERENCES ic_users (id) ON UPDATE CASCADE ON DELETE CASCADE, -- member ID
    paramname VARCHAR(255) NOT NULL, -- parameter name (key)
    paramval LONGTEXT NOT NULL, -- parameter value
    PRIMARY KEY (uid, paramname)
) ENGINE=InnoDB;
